ALTER TABLE ListsParams CHANGE `filter` `filter` TEXT NOT NULL DEFAULT '';
ALTER TABLE DashboardPreset CHANGE type type ENUM('admin', 'reseller', 'client', 'domain', 'private', 'custom') NOT NULL DEFAULT 'private';

-- create ip_pool table
CREATE TABLE IF NOT EXISTS ip_pool (  
	id INT UNSIGNED AUTO_INCREMENT NOT NULL,
	ip_address_id INT(10) UNSIGNED NOT NULL,
	`type` enum('shared','exclusive'),
	PRIMARY KEY (id, ip_address_id)
) TYPE=InnoDB;

-- create Webmails table
CREATE TABLE IF NOT EXISTS `Webmails` (
     `id` int(10) unsigned NOT NULL auto_increment,
    `product_id` varchar(255) character set ascii COLLATE ascii_bin NOT NULL,
    `name` varchar(255) character set utf8 COLLATE utf8_general_ci NOT NULL,
    `version` varchar(255) character set utf8 COLLATE utf8_general_ci NOT NULL,
     `release` varchar(255) character set utf8 COLLATE utf8_general_ci NOT NULL,
    `docroot` varchar(255) character set utf8 COLLATE utf8_general_ci NOT NULL,
    `enabled` enum('false', 'true') NOT NULL default 'false',
    PRIMARY KEY (`id`),
     UNIQUE KEY `product_id` (`product_id`),
	UNIQUE KEY `docroot` (`docroot`)
) TYPE=InnoDB;

-- APS Catalog

CREATE TABLE IF NOT EXISTS APSCatalogUpdates (
    id                  INT UNSIGNED AUTO_INCREMENT PRIMARY KEY
,   `package_name`              VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci
,   `package_version`           VARCHAR(30) CHARACTER SET utf8 COLLATE utf8_general_ci
,   `package_release`           VARCHAR(30) CHARACTER SET utf8 COLLATE utf8_general_ci

,   `update_name`               VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci
,   `update_version`            VARCHAR(30) CHARACTER SET utf8 COLLATE utf8_general_ci
,   `update_release`            VARCHAR(30) CHARACTER SET utf8 COLLATE utf8_general_ci
,   `update_vendor`             VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci
,   `update_packager`           VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci

,   `received`              DATETIME
,   `update_type`               ENUM('patch', 'upgrade') NOT NULL
,   `recommended`               ENUM('true', 'false') NOT NULL DEFAULT 'false'
) TYPE=InnoDB;

CREATE TABLE IF NOT EXISTS longtasks (
    `id`            INT UNSIGNED AUTO_INCREMENT PRIMARY KEY
,   `type`          VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL
,   `title`         VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL
,   `status`        ENUM('not_started', 'cancel_pending', 'canceled', 'error', 'fatal_error', 'done', 'started') NOT NULL DEFAULT 'not_started'
,   `progress_text`     VARCHAR(245) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL
,   `progress_max`      INT UNSIGNED NOT NULL DEFAULT 1
,   `progress_value`    INT UNSIGNED NOT NULL DEFAULT 0
,   `start_time`        TIMESTAMP(14)
,   `finish_time`       TIMESTAMP(14)
,   `owner_type`        SMALLINT UNSIGNED NOT NULL DEFAULT 0
,   `owner`         INT UNSIGNED NOT NULL DEFAULT 0
,   `can_cancel`        ENUM('false', 'true') NOT NULL DEFAULT 'false'
,   `pid`           INT UNSIGNED NOT NULL DEFAULT 0
) TYPE=InnoDB;

CREATE TABLE IF NOT EXISTS longtaskparams (
    `task_id`       INT UNSIGNED NOT NULL
,   `param`         VARCHAR(245) CHARACTER SET ascii COLLATE ascii_general_ci NOT NULL
                    -- parameter name
,   `val`           VARCHAR(255) CHARACTER SET binary
                    -- parameter value
,               PRIMARY KEY (task_id, param)
) TYPE=InnoDB;

-- End of APS Catalog upgrade

-- add reseller actions
replace into actions(name, descr, enabled) values ('reseller_create', 'Create Reseller', 'true');
replace into actions(name, descr, enabled) values ('reseller_update', 'Update Reseller', 'true');
replace into actions(name, descr, enabled) values ('reseller_delete', 'Delete Reseller', 'true');
replace into actions(name, descr, enabled) values ('reseller_status_update', 'Update Reseller Status', 'true');
replace into actions(name, descr, enabled) values ('reseller_preferences_update', 'Update Reseller Preferences', 'true');
replace into actions(name, descr, enabled) values ('reseller_ip_pool_update', 'Update Reseller IP Pool', 'true');
replace into actions(name, descr, enabled) values ('reseller_siteapp_added', 'Site Application Package added to Reseller Pool', 'true');
replace into actions(name, descr, enabled) values ('reseller_siteapp_removed', 'Site Application Package removed from Reseller Pool', 'true');
replace into actions(name, descr, enabled) values ('reseller_limit_size_reached', 'Reseller\'s disk space limit reached', 'true');
replace into actions(name, descr, enabled) values ('reseller_limit_traffic_reached', 'Reseller\'s traffic limit reached', 'true');
replace into actions(name, descr, enabled) values ('reseller_limits_update', 'Update Reseller Limits', 'true');
replace into actions(name, descr, enabled) values ('reseller_permissions_update', 'Update Reseller Permissions', 'true');
replace into actions(name, descr, enabled) values ('reseller_guid_update', 'Change Reseller GUID', 'true');

-- upgrade tts_reporters
ALTER TABLE `tts_reporters` MODIFY `type` ENUM('client','domain','mailname','email','admin','reseller') NOT NULL default 'email';

-- upgrade DashboardPreset
ALTER TABLE `DashboardPreset` DROP INDEX name_type;
ALTER TABLE `DashboardPreset` ADD UNIQUE name_type (name, type);
ALTER TABLE `DashboardPresetConfig` MODIFY `type` ENUM('none', 'button', 'custom-buttons', 'top', 'column', 'footer', 'indicator', 'left', 'right', 'section', 'shortcut', 'tab', 'tabs', 'quick-search', 'quick-search-option') NOT NULL default 'none';

REPLACE INTO DashboardPreset (uri, name, type) values ('/plesk/dashboard/', 'Default Administrator Home Page', 'admin');
SET @id = LAST_INSERT_ID();
REPLACE INTO misc (param, val) VALUES ('def_admin_preset_id', @id);
REPLACE INTO misc (param, val) VALUES ('admin_preset_id', @id);

REPLACE INTO DashboardPreset (uri, name, type) values ('/plesk/client@/dashboard/', 'Default Client Home Page', 'client');
SET @id = LAST_INSERT_ID();
REPLACE INTO misc (param, val) VALUES ('def_client_preset_id', @id);

REPLACE INTO DashboardPreset (uri, name, type) values ('/plesk/client@/domain@/dashboard/', 'Default Domain Owner Home Page', 'domain');
SET @id = LAST_INSERT_ID();
REPLACE INTO misc (param, val) VALUES ('def_domain_preset_id', @id);

REPLACE INTO DashboardPreset (uri, name, type) values ('/plesk/reseller@/dashboard/', 'Default Reseller Home Page', 'reseller');
SET @id = LAST_INSERT_ID();
REPLACE INTO misc (param, val) VALUES ('def_reseller_preset_id', @id);

-- custom buttons

ALTER TABLE custom_buttons CHANGE place place ENUM('domain', 'client', 'reseller', 'admin', 'navigation') NOT NULL DEFAULT 'domain'; 

-- default parameters for clients

REPLACE INTO cl_param (cl_id, param, val) SELECT id, 'OveruseBlock', 'true' FROM clients;
REPLACE INTO cl_param (cl_id, param, val) SELECT id, 'OveruseNotify', 'true' FROM clients;

-- default parameters for domains

REPLACE INTO dom_param (dom_id, param, val) SELECT id, 'OveruseBlock', 'true' FROM domains;
REPLACE INTO dom_param (dom_id, param, val) SELECT id, 'OveruseNotify', 'true' FROM domains;

-- Billing integration
CREATE TABLE IF NOT EXISTS BillingCache (
	GUID			CHAR(36) CHARACTER SET ascii COLLATE ascii_bin NOT NULL PRIMARY KEY
								-- global uniq identifier GUID
,	BALANCE			VARCHAR(20) CHARACTER SET ascii COLLATE ascii_bin
								-- plesk's client balance in a billing
,	CURRENCY		CHAR(3) CHARACTER SET ascii COLLATE ascii_bin NOT NULL DEFAULT 'USD'
								-- currency of the balance
,	INVOICEDATEDUE	TIMESTAMP(14)
								-- due date
,	CLIENTID		INT(8) UNSIGNED
								-- billing system's client ID
) TYPE=InnoDB;

-- PersistentCache table
CREATE TABLE IF NOT EXISTS PersistentCache (
	`key`						VARCHAR(128) CHARACTER SET ascii COLLATE ascii_general_ci NOT NULL
,	`uid`						VARCHAR(32) CHARACTER SET ascii COLLATE ascii_general_ci NOT NULL
,	`expire`				TIMESTAMP
, `events`				VARCHAR(255) CHARACTER SET ascii COLLATE ascii_general_ci NOT NULL DEFAULT ''
, PRIMARY KEY (`key`, `uid`)
) TYPE=InnoDB;

-- BackendCache table
CREATE TABLE IF NOT EXISTS BackendCache (
	`key`						VARCHAR(128) CHARACTER SET ascii COLLATE ascii_general_ci NOT NULL
,	`data`						BLOB NOT NULL DEFAULT ''
, PRIMARY KEY (`key`)
) TYPE=InnoDB;